﻿using System.Text;

using Org.BouncyCastle.Crypto.Generators;
using Org.BouncyCastle.Crypto.Parameters;
using Org.BouncyCastle.Crypto.Signers;
using Org.BouncyCastle.Security;

namespace PqcAlmanac;

/**
 * Example of ML-DSA signature generation using the ML-DSA-65 parameter set.
 */
public class MLDsaExample
{
    private static byte[] Msg = Encoding.UTF8.GetBytes("Hello, world!");

    public static int Main(string[] args)
    {
        var random = new SecureRandom();

        // Generate ML-DSA key pair.
        var kpg = new MLDsaKeyPairGenerator();
        kpg.Init(new MLDsaKeyGenerationParameters(random, MLDsaParameters.ml_dsa_65));
        var kp = kpg.GenerateKeyPair();

        // Create ML-DSA signer.
        var signer = SignerUtilities.InitSigner("ML-DSA-65", forSigning: true, kp.Private, random);

        // Generate ML-DSA signature.
        signer.BlockUpdate(Msg, 0, Msg.Length);
        byte[] signature = signer.GenerateSignature();

        // Verify ML-DSA signature.
        var verifier = SignerUtilities.InitSigner("ML-DSA-65", forSigning: false, kp.Public, random: null);

        verifier.BlockUpdate(Msg, 0, Msg.Length);
        if (verifier.VerifySignature(signature))
        {
            Console.WriteLine("ML-DSA-65 signature created and verified successfully");
            return 0;
        }

        return 1;
    }
}
