﻿using Org.BouncyCastle.Asn1.X509;
using Org.BouncyCastle.Crypto.Generators;
using Org.BouncyCastle.Crypto.Operators;
using Org.BouncyCastle.Crypto.Parameters;
using Org.BouncyCastle.Pkcs;
using Org.BouncyCastle.Security;

namespace PqcAlmanac;

/**
* Create a basic PKCS#10 request using ML-DSA-44.
*/
public class MLDsaPkcs10Example
{
    public static int Main(string[] args)
    {
        var random = new SecureRandom();

        // Generate an ML-DSA-44 key pair.
        var kpg = new MLDsaKeyPairGenerator();
        kpg.Init(new MLDsaKeyGenerationParameters(random, MLDsaParameters.ml_dsa_44));
        var keyPair = kpg.GenerateKeyPair();

        // Create PKCS#10 request
        var signatureFactory = new Asn1SignatureFactory("ML-DSA-44", keyPair.Private);
        var subject = new X509Name("CN=ML-DSA Certification Request");
        var pkcs10 = new Pkcs10CertificationRequest(signatureFactory, subject, keyPair.Public, attributes: null);

        // Verify signature
        if (pkcs10.Verify(keyPair.Public))
        {
            Console.WriteLine("ML-DSA PKCS#10 request verified");
            return 0;
        }

        return 1;
    }
}
